@@ -4,6 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010, Alan Calvert
+    Copyright 2018, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -19,7 +20,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified October 2010
+    This file is derivative of ZynAddSubFX original code.
+
+    Modified February 2018
 */
 
 #include "Misc/SynthEngine.h"
@@ -40,7 +43,8 @@ EQ::EQ(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
         filter[i].r = new AnalogFilter(6, 1000.0, 1.0, 0, synth);
     }
     // default values
-    Pvolume = 50;
+    setvolume(50);
+    Pband = 0;
     setpreset(Ppreset);
     cleanup();
 }
@@ -56,20 +60,53 @@ void EQ::cleanup(void)
     }
 }
 
+
 // Effect output
 void EQ::out(float *smpsl, float *smpsr)
 {
-    memcpy(efxoutl, smpsl, synth->p_bufferbytes);
-    memcpy(efxoutr, smpsr, synth->p_bufferbytes);
-    for (int i = 0; i < synth->p_buffersize; ++i)
+    memcpy(efxoutl, smpsl, synth->sent_bufferbytes);
+    memcpy(efxoutr, smpsr, synth->sent_bufferbytes);
+    for (int i = 0; i < synth->sent_buffersize; ++i)
     {
-        efxoutl[i] *= volume;
-        efxoutr[i] *= volume;
+        efxoutl[i] *= volume.getValue();
+        efxoutr[i] *= volume.getValue();
+        volume.advanceValue();
     }
     for (int i = 0; i < MAX_EQ_BANDS; ++i)
     {
         if (filter[i].Ptype == 0)
             continue;
+
+        float oldval = filter[i].freq.getValue();
+        filter[i].freq.advanceValue(synth->sent_buffersize);
+        float newval = filter[i].freq.getValue();
+        if (oldval != newval) {
+            filter[i].l->interpolatenextbuffer();
+            filter[i].l->setfreq(newval);
+            filter[i].r->interpolatenextbuffer();
+            filter[i].r->setfreq(newval);
+        }
+
+        oldval = filter[i].gain.getValue();
+        filter[i].gain.advanceValue(synth->sent_buffersize);
+        newval = filter[i].gain.getValue();
+        if (oldval != newval) {
+            filter[i].l->interpolatenextbuffer();
+            filter[i].l->setgain(newval);
+            filter[i].r->interpolatenextbuffer();
+            filter[i].r->setgain(newval);
+        }
+
+        oldval = filter[i].q.getValue();
+        filter[i].q.advanceValue(synth->sent_buffersize);
+        newval = filter[i].q.getValue();
+        if (oldval != newval) {
+            filter[i].l->interpolatenextbuffer();
+            filter[i].l->setq(newval);
+            filter[i].r->interpolatenextbuffer();
+            filter[i].r->setq(newval);
+        }
+
         filter[i].l->filterout(efxoutl);
         filter[i].r->filterout(efxoutr);
     }
@@ -81,8 +118,9 @@ void EQ::out(float *smpsl, float *smpsr)
 void EQ::setvolume(unsigned char Pvolume_)
 {
     Pvolume = Pvolume_;
-    outvolume = powf(0.005f, (1.0f - Pvolume / 127.0f)) * 10.0f;
-    volume = (!insertion) ? 1.0f : outvolume;
+    float tmp = powf(0.005f, (1.0f - Pvolume / 127.0f)) * 10.0f;
+    outvolume.setTargetValue(tmp);
+    volume.setTargetValue((!insertion) ? 1.0f : tmp);
 }
 
 
@@ -110,6 +148,8 @@ void EQ::changepar(int npar, unsigned char value)
         case 0:
             setvolume(value);
             break;
+        case 1:
+            Pband = value;
     }
     if (npar < 10)
         return;
@@ -132,24 +172,25 @@ void EQ::changepar(int npar, unsigned char value)
                 filter[nb].r->settype(value - 1);
             }
             break;
+
         case 1:
             filter[nb].Pfreq = value;
             tmp = 600.0f * powf(30.0f, (value - 64.0f) / 64.0f);
-            filter[nb].l->setfreq(tmp);
-            filter[nb].r->setfreq(tmp);
+            filter[nb].freq.setTargetValue(tmp);
             break;
+
         case 2:
             filter[nb].Pgain = value;
             tmp = 30.0f * (value - 64.0f) / 64.0f;
-            filter[nb].l->setgain(tmp);
-            filter[nb].r->setgain(tmp);
+            filter[nb].gain.setTargetValue(tmp);
             break;
+
         case 3:
             filter[nb].Pq = value;
             tmp = powf(30.0f, (value - 64.0f) / 64.0f);
-            filter[nb].l->setq(tmp);
-            filter[nb].r->setq(tmp);
+            filter[nb].q.setTargetValue(tmp);
             break;
+
         case 4:
             filter[nb].Pstages = value;
             if (value >= MAX_FILTER_STAGES)
@@ -160,6 +201,7 @@ void EQ::changepar(int npar, unsigned char value)
     }
 }
 
+
 unsigned char EQ::getpar(int npar)
 {
     switch (npar)
@@ -167,6 +209,8 @@ unsigned char EQ::getpar(int npar)
         case 0:
             return Pvolume;
             break;
+        case 1:
+            return Pband;
     }
     if (npar < 10)
         return 0;
@@ -180,15 +224,19 @@ unsigned char EQ::getpar(int npar)
         case 0:
             return(filter[nb].Ptype);
             break;
+
         case 1:
             return(filter[nb].Pfreq);
             break;
+
         case 2:
             return(filter[nb].Pgain);
             break;
+
         case 3:
             return(filter[nb].Pq);
             break;
+
         case 4:
             return(filter[nb].Pstages);
             break;
@@ -206,5 +254,70 @@ float EQ::getfreqresponse(float freq)
             continue;
         resp *= filter[i].l->H(freq);
     }
-    return rap2dB(resp * outvolume);
+    // Only for UI purposes, use target value.
+    return rap2dB(resp * outvolume.getTargetValue());
 }
+
+
+float EQlimit::getlimits(CommandBlock *getData)
+{
+    int value = getData->data.value;
+    int control = getData->data.control;
+    int request = getData->data.type & 3; // clear upper bits
+
+    int min = 0;
+    int max = 127;
+    int def = 0;
+    bool canLearn = true;
+    bool isInteger = true;
+
+    switch (control)
+    {
+        case 0:
+            def = 64;
+            break;
+        case 1:
+            max = 7;
+            canLearn = false;
+            break;
+        case 10:
+            max = 9;
+            canLearn = false;
+            break;
+        case 11:
+        case 12:
+        case 13:
+            def = 64;
+            break;
+        case 14:
+            max = 4;
+            canLearn = false;
+            break;
+        default:
+            getData->data.type |= 4; // error
+            return 1.0f;
+            break;
+    }
+
+    switch(request)
+    {
+        case 0:
+            if(value < min)
+                value = min;
+            else if(value > max)
+                value = max;
+            break;
+        case 1:
+            value = min;
+            break;
+        case 2:
+            value = max;
+            break;
+        case 3:
+            value = def;
+            break;
+    }
+    getData->data.type |= (canLearn * 64 + isInteger * 128);
+    return float(value);
+}
+
