@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010, Alan Calvert
-    Copyright 2014-2015, Will Godfrey & others
+    Copyright 2014-2018, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -20,14 +20,18 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of a ZynAddSubFX original, last modified January 2015
+    This file is a derivative of a ZynAddSubFX original.
+
+    Modified June 2018
 */
 
 #include <set>
 #include <list>
 #include <stdlib.h>
+#include <stdio.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <iostream>
 
 #include <sys/types.h>
 #include <fcntl.h>
@@ -48,6 +52,7 @@ using namespace std;
 Bank::Bank(SynthEngine *_synth) :
     defaultinsname(string(" ")),
     xizext(".xiz"),
+    xiyext(".xiy"),
     force_bank_dir_file(".bankdir"), // if this file exists in a directory, the
                                     // directory is considered a bank, even if
                                     // it doesn't contain an instrument file
@@ -56,7 +61,6 @@ Bank::Bank(SynthEngine *_synth) :
     currentBankID(0)
 {
     roots.clear();
-    //addDefaultRootDirs();
 }
 
 
@@ -79,11 +83,15 @@ string Bank::getRootFileTitle()
 
 
 // Get the name of an instrument from the bank
-string Bank::getname(unsigned int ninstrument)
+string Bank::getname(unsigned int ninstrument, size_t bank, size_t root)
 {
-    if (emptyslot(ninstrument))
+    if (root == 0xff)
+        root = currentRootID;
+    if (bank == 0xff)
+        bank = currentBankID;
+    if (emptyslotWithID(root, bank, ninstrument))
         return defaultinsname;
-    return getInstrumentReference(ninstrument).name;
+    return getInstrumentReference(root, bank, ninstrument).name;
 }
 
 
@@ -109,28 +117,58 @@ string Bank::getnamenumbered(unsigned int ninstrument)
 
 
 // Changes the name of an instrument (and the filename)
-bool Bank::setname(unsigned int ninstrument, string newname, int newslot)
+bool Bank::setname(unsigned int ninstrument, string newname, int newslot, size_t oldBank, size_t newBank, size_t oldRoot, size_t newRoot)
 {
-    if (emptyslot(ninstrument))
+    if (oldBank == 0xff)
+        oldBank = currentBankID;
+    if (oldRoot == 0xff)
+        oldRoot = currentRootID;
+    if (newBank == 0xff)
+        newBank = oldBank;
+    if (newRoot == 0xff)
+        newRoot = oldRoot;
+
+    if (emptyslotWithID(oldRoot, oldBank, ninstrument))
         return false;
 
+    string newfilepath = getBankPath(newRoot, newBank);
+    if (newfilepath.at(newfilepath.size() - 1) != '/')
+        newfilepath += "/";
+
     int slot = (newslot >= 0) ? newslot + 1 : ninstrument + 1;
     string filename = "0000" + asString(slot);
+
     filename = filename.substr(filename.size() - 4, 4) + "-" + newname + xizext;
     legit_filename(filename);
-    string newfilepath = getBankPath(currentRootID, currentBankID);
-    if (newfilepath.at(newfilepath.size() - 1) != '/')
-        newfilepath += "/";
+
+    int chk = -1;
+    int chk2 = -1;
     newfilepath += filename;
-    InstrumentEntry &instrRef = getInstrumentReference(currentRootID, currentBankID, ninstrument);
-    int chk = rename(getFullPath(currentRootID, currentBankID, ninstrument).c_str(), newfilepath.c_str());
-    if (chk < 0)
+    string oldfilepath = setExtension(getFullPath(oldRoot, oldBank, ninstrument), xizext);
+    chk = rename(oldfilepath.c_str(), newfilepath.c_str());
+    /*if (chk < 0)
     {
         synth->getRuntime().Log("setName failed renaming "
-                    + getFullPath(currentRootID, currentBankID, ninstrument) + " -> "
-                    + newfilepath + ": " + string(strerror(errno)));
+                + oldfilepath + " -> "
+                + newfilepath + ": " + string(strerror(errno)));
+    }*/
+
+    newfilepath = setExtension(newfilepath, xiyext);
+    oldfilepath = setExtension(oldfilepath, xiyext);
+    chk2 = rename(oldfilepath.c_str(), newfilepath.c_str());
+    /*if (chk2 < 0)
+    {
+        synth->getRuntime().Log("setName failed renaming "
+                + oldfilepath + " -> "
+                + newfilepath + ": " + string(strerror(errno)));
+    }*/
+
+    if (chk < 0 && chk2 < 0)
+    {
+        synth->getRuntime().Log("setName failed renaming " + oldfilepath + " -> " + newfilepath + ": " + string(strerror(errno)));
         return false;
     }
+    InstrumentEntry &instrRef = getInstrumentReference(oldRoot, oldBank, ninstrument);
     instrRef.name = newname;
     instrRef.filename = filename;
     return true;
@@ -154,38 +192,49 @@ bool Bank::emptyslotWithID(size_t rootID, size_t bankID, unsigned int ninstrumen
 // Removes the instrument from the bank
 bool Bank::clearslot(unsigned int ninstrument)
 {
+    int chk = 0;
+    int chk2 = 0; // to stop complaints
     if (emptyslot(ninstrument))
         return true;
-    int chk = remove(getFullPath(currentRootID, currentBankID, ninstrument).c_str());
-    if (chk < 0)
+    string tmpfile = setExtension(getFullPath(currentRootID, currentBankID, ninstrument), xiyext);
+
+    if (isRegFile(tmpfile))
     {
-        synth->getRuntime().Log(asString(ninstrument) + " Failed to remove "
-                     + getFullPath(currentRootID, currentBankID, ninstrument) + " "
-                     + string(strerror(errno)));
-        return false;
+        chk = remove(tmpfile.c_str());
+        if (chk < 0)
+            synth->getRuntime().Log(asString(ninstrument) + " Failed to remove " + tmpfile);
     }
+    tmpfile = setExtension(tmpfile, xizext);
+    if (isRegFile(tmpfile))
+    {
+        chk2 = remove(tmpfile.c_str());
+        if (chk2 < 0)
+            synth->getRuntime().Log(asString(ninstrument) + " Failed to remove " + tmpfile);
+    }
+    if (chk < 0 || chk2 < 0)
+        return false;
+
     deletefrombank(currentRootID, currentBankID, ninstrument);
     return true;
 }
 
 
-// Save the instrument to a slot
-bool Bank::savetoslot(unsigned int ninstrument, Part *part)
+bool Bank::savetoslot(size_t rootID, size_t bankID, int ninstrument, int npart)
 {
-    if (ninstrument >= BANK_SIZE)
-    {
-        synth->getRuntime().Log("Can't save " + asString(ninstrument) + ", slot > bank size");
-        return false;
-    }
+    string filepath = getBankPath(rootID, bankID);
+    string name = synth->part[npart]->Pname;
+    if (filepath.at(filepath.size() - 1) != '/')
+        filepath += "/";
     clearslot(ninstrument);
     string filename = "0000" + asString(ninstrument + 1);
     filename = filename.substr(filename.size() - 4, 4)
-               + "-" + part->Pname + xizext;
+               + "-" + name + xizext;
     legit_filename(filename);
-    string filepath = getBankPath(currentRootID, currentBankID);
-    if (filepath.at(filepath.size() - 1) != '/')
-        filepath += "/";
+
     string fullpath = filepath + filename;
+    bool ok1 = true;
+    bool ok2 = true;
+    int saveType = synth->getRuntime().instrumentFormat;
     if (isRegFile(fullpath))
     {
         int chk = remove(fullpath.c_str());
@@ -196,23 +245,43 @@ bool Bank::savetoslot(unsigned int ninstrument, Part *part)
             return false;
         }
     }
-    if (!part->saveXML(fullpath))
+    if (saveType & 1) // legacy
+        ok2 = synth->part[npart]->saveXML(fullpath, false);
+
+    fullpath = setExtension(fullpath, xiyext);
+    if (isRegFile(fullpath))
+    {
+        int chk = remove(fullpath.c_str());
+        if (chk < 0)
+        {
+            synth->getRuntime().Log("saveToSlot failed to unlink " + fullpath
+                        + ", " + string(strerror(errno)));
+            return false;
+        }
+    }
+
+    if (saveType & 2) // Yoshimi format
+        ok1 = synth->part[npart]->saveXML(fullpath, true);
+    if (!ok1 || !ok2)
         return false;
+
     filepath += force_bank_dir_file;
     FILE *tmpfile = fopen(filepath.c_str(), "w+");
     fputs (YOSHIMI_VERSION, tmpfile);
     fclose(tmpfile);
-    addtobank(currentRootID, currentBankID, ninstrument, filename, part->Pname);
+    addtobank(rootID, bankID, ninstrument, filename, name);
     return true;
 }
 
 
 //Gets a bank name
-string Bank::getBankName(int bankID)
+string Bank::getBankName(int bankID, size_t rootID)
 {
-    if (roots [currentRootID].banks.count(bankID) == 0)
+    if (rootID > 0x7f)
+        rootID = currentRootID;
+    if (roots [rootID].banks.count(bankID) == 0)
         return "";
-    return string(roots [currentRootID].banks [bankID].dirname);
+    return string(roots [rootID].banks [bankID].dirname);
 }
 
 
@@ -227,6 +296,22 @@ string Bank::getBankIDname(int bankID)
 }
 
 
+bool Bank::isDuplicateBankName(size_t rootID, string name)
+{
+    //if(roots.count(rootID) == 0)
+        //return false;
+    for (int i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
+    {
+        string check = getBankName(i,rootID);
+        if (check > "" && check == name)
+            return true;
+        if (check > "")
+            cout << check << endl;
+    }
+    return false;
+}
+
+
 // finds the number of instruments in a bank
 int Bank::getBankSize(int bankID)
 {
@@ -280,7 +365,6 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
     roots [rootID].banks [banknum].instruments.clear();
 
     struct dirent *fn;
-    struct stat st;
     string chkpath;
     string candidate;
     size_t xizpos;
@@ -295,10 +379,16 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
         if (chkpath.at(chkpath.size() - 1) != '/')
             chkpath += "/";
         chkpath += candidate;
-        lstat(chkpath.c_str(), &st);
-        if (S_ISREG(st.st_mode))
+        if (isRegFile(chkpath))
         {
-            if ((xizpos = candidate.rfind(xizext)) != string::npos)
+            if (chkpath.rfind(".xiz") != string::npos && isRegFile(setExtension(chkpath, xiyext)))
+                continue; // don't want .xiz if there is .xiy
+
+            xizpos = candidate.rfind(".xiy");
+            if (xizpos == string::npos)
+                xizpos = candidate.rfind(xizext);
+
+            if (xizpos != string::npos)
             {
                 if (xizext.size() == (candidate.size() - xizpos))
                 {
@@ -306,7 +396,6 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
                     // sa verific daca e si extensia dorita
 
                     // sorry Cal. They insisted :(
-
                     int chk = findSplitPoint(candidate);
                     if (chk > 0)
                     {
@@ -330,10 +419,195 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
 }
 
 
-// Makes a new bank with known ID. Does *not* make it current
-bool Bank::newIDbank(string newbankdir, unsigned int bankID)
+// Creates an external bank and copies in the contents of the IDd one
+unsigned int Bank::exportBank(string exportdir, size_t rootID, unsigned int bankID)
 {
-    if (!newbankfile(newbankdir))
+    if (rootID > 0x7f)
+        rootID = currentRootID;
+    string name = "";
+    string sourcedir = "";
+    bool ok = true;
+    if (roots.count(rootID) == 0)
+    {
+        name = "Root ID " + to_string(int(rootID)) + " doesn't exist";
+        ok = false;
+    }
+    if (ok && roots [rootID].banks.count(bankID) == 0)
+    {
+        name = "Bank " + to_string(bankID) + " is empty";
+        ok = false;
+    }
+    else
+        sourcedir = getRootPath(rootID) + "/" + getBankName(bankID, rootID);
+
+    if (ok && isDirectory(exportdir))
+    {
+        ok = false;
+        name = "Can't overwrite existing directory";
+    }
+    if (ok)
+    {
+        int result = mkdir(exportdir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
+        if (result < 0)
+        {
+            name = "Can't create external bank " + findleafname(exportdir);
+            ok = false;
+        }
+        else
+        {
+            DIR *dir = opendir(sourcedir.c_str());
+            struct dirent *fn;
+            int count = 0;
+            int missing = 0;
+            while ((fn = readdir(dir)))
+            {
+                string nextfile = string(fn->d_name);
+                if (copyFile(sourcedir + "/" + nextfile, exportdir + "/" + nextfile))
+                    ++missing;
+                else
+                    ++count;
+            }
+            if (count > 0)
+            {
+                name = "Copied out " + to_string(count) + " files to " + exportdir + " ";
+                if (missing > 2) // seem to get 2 phantoms :(
+                    name +=( "but failed to transfer" + to_string(missing));
+            }
+            else
+            {
+                name = "to transfer to " + exportdir; // failed
+                ok = false;
+            }
+        }
+    }
+
+    unsigned int msgID = miscMsgPush(name);
+    if (!ok)
+        msgID |= 0x1000;
+    return msgID;
+}
+
+
+// Creates a new bank and copies in the contents of the external one
+unsigned int Bank::importBank(string importdir, size_t rootID, unsigned int bankID)
+{
+    if (rootID > 0x7f)
+        rootID = currentRootID;
+    string name = "";
+    bool ok = true;
+    if (roots.count(rootID) == 0)
+    {
+        name = "Root ID " + to_string(int(rootID)) + " doesn't exist";
+        ok = false;
+    }
+
+    if (ok && roots [rootID].banks.count(bankID) != 0)
+    {
+        name = "Bank " + to_string(bankID) + " already contains " + getBankName(bankID, rootID);
+        ok = false;
+    }
+
+    if (ok)
+    {
+        DIR *dir = opendir(importdir.c_str());
+        if (dir == NULL)
+        {
+            name = "Can't find " + importdir;
+            ok = false;
+        }
+        else
+        {
+            string bankname = findleafname(importdir);
+            int repeats = 0;
+            string suffix = "";
+            while (isDirectory(getRootPath(rootID) + "/" + bankname + suffix))
+            {
+                ++repeats;
+                suffix = "~" + to_string(repeats);
+                //cout << ">" << suffix << endl;
+            }
+            bankname += suffix;
+            //cout << bankname << endl;
+            if (!newIDbank(bankname, bankID, rootID))
+            {
+                name = "Can't create bank " + bankname;
+                ok = false;
+            }
+            else
+            {
+                int count = 0;
+                int total = -2; // seem to get 2 phantoms :(
+                bool missing = false;
+                struct dirent *fn;
+                string exportfile = getRootPath(rootID) + "/" + getBankName(bankID, rootID);
+                while ((fn = readdir(dir)))
+                {
+                    string nextfile = string(fn->d_name);                    if (nextfile.rfind(".bankdir") != string::npos)
+                        continue; // new version will be generated
+                    ++total;
+                    if (nextfile.rfind(".xiy") != string::npos || nextfile.rfind(".xiz") != string::npos)
+                    {
+                        ++count;
+                        int pos = -1; // default for un-numbered
+                        int slash = nextfile.rfind("/") + 1;
+                        int hyphen = nextfile.rfind("-");
+                        if (hyphen > slash && (hyphen - slash) <= 4)
+                            pos = stoi(nextfile.substr(slash, hyphen)) - 1;
+
+                        if (copyFile(importdir + "/" + nextfile, exportfile + "/" + nextfile))
+                            missing = true;
+                        string stub;
+                        if (pos >= -1)
+                            stub = findleafname(nextfile).substr(hyphen + 1);
+                        else
+                            stub = findleafname(nextfile);
+                        if (!isDuplicate(rootID, bankID, pos, nextfile))
+                        {
+                            if (addtobank(rootID, bankID, pos, nextfile, stub))
+                                missing = true;
+                        }
+                    }
+                }
+                name = importdir;
+                if (count == 0)
+                    name += " : No valid instruments found";
+                else if (missing)
+                    name += " : Failed to copy some instruments";
+                else if (count < total)
+                    name = name + " : Ignored " + to_string(total - count)  + " unrecognised items";
+            }
+        }
+    }
+    unsigned int msgID = miscMsgPush(name);
+    if (!ok)
+        msgID |= 0x1000;
+    return msgID;
+}
+
+
+bool Bank::isDuplicate(size_t rootID, size_t bankID, int pos, const string filename)
+{
+    //cout << filename << " count " << roots [rootID].banks.count(bankID) << endl;
+    string path = getRootPath(rootID) + "/" + getBankName(bankID, rootID) + "/" + filename;
+    if (isRegFile(setExtension(path, xiyext)) && filename.rfind(xizext) < string::npos)
+        return 1;
+    if (isRegFile(setExtension(path, xizext)) && filename.rfind(xiyext) < string::npos)
+    {
+        InstrumentEntry &Ref = getInstrumentReference(rootID, bankID, pos);
+        Ref.yoshiType = true;
+        return 1;
+    }
+    return 0;
+}
+
+
+// Makes a new bank with known ID. Does *not* make it current
+bool Bank::newIDbank(string newbankdir, unsigned int bankID, size_t rootID)
+{
+    if (rootID > 0x7f)
+        rootID = currentRootID; // should be needed!
+
+    if (!newbankfile(newbankdir, rootID))
         return false;
     roots [currentRootID].banks [bankID].dirname = newbankdir;
     hints [currentRootID] [newbankdir] = bankID; // why do we need this?
@@ -342,14 +616,14 @@ bool Bank::newIDbank(string newbankdir, unsigned int bankID)
 
 
 // Performs the actual file operation for new banks
-bool Bank::newbankfile(string newbankdir)
+bool Bank::newbankfile(string newbankdir, size_t rootID)
 {
      if (getRootPath(currentRootID).empty())
     {
         synth->getRuntime().Log("Current bank root directory not set");
         return false;
     }
-    string newbankpath = getRootPath(currentRootID);
+    string newbankpath = getRootPath(rootID);
     if (newbankpath.at(newbankpath.size() - 1) != '/')
         newbankpath += "/";
     newbankpath += newbankdir;
@@ -373,151 +647,302 @@ bool Bank::newbankfile(string newbankdir)
 
 
 // Removes a bank and all its contents
-bool Bank::removebank(unsigned int bankID)
+unsigned int Bank::removebank(unsigned int bankID, size_t rootID)
 {
-    int chk;
+    int chk = 0;
+    if (rootID == 255)
+        rootID = currentRootID;
+    if (roots.count(rootID) == 0)
+    {
+        chk = 0x1000 | miscMsgPush("Root " + to_string(int(rootID)) + " is empty!");
+        return chk;
+    }
+    string bankName = getBankPath(rootID, bankID);
+    string IDfile = bankName + "/.bankdir";
+    FILE *tmpfile = fopen(IDfile.c_str(), "w+");
+    if (!tmpfile)
+        chk = 0x1000 | miscMsgPush("Can't delete from this location.");
+    else
+        fclose(tmpfile);
 
+    int ck1 = 0;
+    int ck2 = 0;
+    string name;
     for (int inst = 0; inst < BANK_SIZE; ++ inst)
     {
-        if (!roots [currentRootID].banks [bankID].instruments [inst].name.empty())
+        if (!roots [rootID].banks [bankID].instruments [inst].name.empty())
         {
-            chk = remove(getFullPath(currentRootID, bankID, inst).c_str());
-            if (chk < 0)
-            {
-                synth->getRuntime().Log(asString(inst) + " Failed to remove "
-                                        + getFullPath(currentRootID, bankID, inst) + " "
-                                        + string(strerror(errno)), 2);
-                return false;
-            }
-            deletefrombank(currentRootID, bankID, inst);
+            name = setExtension(getFullPath(currentRootID, bankID, inst), xiyext);
+            if (isRegFile(name))
+                ck1 = remove(name.c_str());
+            else
+                ck1 = 0;
+
+            name = setExtension(name, xizext);
+            if (isRegFile(name))
+                ck2 = remove(name.c_str());
+            else
+                ck2 = 0;
+
+            if (ck1 == 0 && ck2 == 0)
+                deletefrombank(rootID, bankID, inst);
+            else if (chk == 0) // only want to name one entry
+                    chk = 0x1000 | miscMsgPush(findleafname(name) + ". Others may also still exist.");
         }
     }
-    string tmp = getBankPath(currentRootID, bankID)+"/.bankdir";
-    if (!access(tmp.c_str(), W_OK))
-    {
-        chk = remove(tmp.c_str());
-        if (chk < 0)
+    if (chk > 0)
+        return chk;
+
+    if (isRegFile(IDfile))
+    { // only removed when bank cleared
+        if (remove(IDfile.c_str()) != 0)
         {
-            synth->getRuntime().Log("Failed to remove bank ID file "
-                                    + string(strerror(errno)), 2);
-            return false;
+            chk = 0x1000 | miscMsgPush(findleafname(name));
+            return chk;
         }
     }
-    chk = remove(getBankPath(currentRootID, bankID).c_str());
-    if (chk < 0)
+
+    if (remove(bankName.c_str()) != 0)
     {
-        synth->getRuntime().Log("Failed to remove bank"
-                                + asString(bankID) + ": "
-                                + string(strerror(errno)), 2);
-        return false;
+        chk = 0x1000 | miscMsgPush(bankName + ". Unrecognised contents still exist.");
+        return chk;
     }
-    roots [currentRootID].banks.erase(bankID);
-    return true;
+
+    roots [rootID].banks.erase(bankID);
+    if (rootID == currentRootID && bankID == currentBankID)
+        setCurrentBankID(0);
+    chk = miscMsgPush(bankName);
+    return chk;
 }
 
 
 // Swaps a slot with another
-bool Bank::swapslot(unsigned int n1, unsigned int n2)
+unsigned int Bank::swapslot(unsigned int n1, unsigned int n2, size_t bank1, size_t bank2, size_t root1, size_t root2)
 {
-    if (n1 == n2)
+    unsigned int result = 0;
+    if (n1 == n2 && bank1 == bank2 && root1 == root2)
         return true;
+    if (bank1 == 255)
+        bank1 = currentBankID;
+    if (bank2 == 255)
+        bank2 = bank1;
+    if (root1 == 255)
+        root1 = currentRootID;
+    if (root2 == 255)
+        root2 = root1;
+    //cout << "\nswap 1 I " << int(n1) << "  B " << int(bank1) << "  R " << int(root1) << endl;
+    //cout << "swap 2 I " << int(n2) << "  B " << int(bank2) << "  R " << int(root2) << endl;
+    string message = "";
+    if (emptyslotWithID(root1, bank1, n1) && emptyslotWithID(root2, bank2, n2))
+        message = "nothing to swap!";
 
-    if (emptyslot(n1) && emptyslot(n2))
-        return true;
-    if (emptyslot(n1)) // make the empty slot the destination
+    else if (emptyslotWithID(root1, bank1, n1)) // make the empty slot the destination
     {
-        if (!setname(n2, getname(n2), n1))
-            return false;
-        getInstrumentReference(n1) = getInstrumentReference(n2);
-        getInstrumentReference(n2).clear();
+        if (!setname(n2, getname(n2, bank2, root2), n1, bank2, bank1, root2, root1))
+        {
+            //cout << "here1 " << getname(n2, bank2, root2) << endl;
+            message = "can't write to " + getname(n2, bank2, root2);
+        }
+        getInstrumentReference(root1, bank1, n1) = getInstrumentReference(root2, bank2, n2);
+        getInstrumentReference(root2, bank2, n2).clear();
     }
-    else if (emptyslot(n2)) // this is just a movement to an empty slot
+    else if (emptyslotWithID(root2, bank2, n2)) // this is just a movement to an empty slot
     {
-        if (!setname(n1, getname(n1), n2))
-            return false;
-        getInstrumentReference(n2) = getInstrumentReference(n1);
-        getInstrumentReference(n1).clear();
+        if (!setname(n1, getname(n1, bank1, root1), n2, bank1, bank2, root1, root2))
+        {
+            //cout << "here2 " << getname(n1, bank1, root1) << endl;
+            message = "can't write to " + getname(n1, bank1, root1);
+        }
+        getInstrumentReference(root2, bank2, n2) = getInstrumentReference(root1, bank1, n1);
+        getInstrumentReference(root1, bank1, n1).clear();
     }
+    else if (bank1 != bank2 || root1 != root2)
+        message = "move only, across banks/roots";
     else
     {   // if both slots are used
-        InstrumentEntry &instrRef1 = getInstrumentReference(n1);
-        InstrumentEntry &instrRef2 = getInstrumentReference(n2);
+        InstrumentEntry &instrRef1 = getInstrumentReference(root1, bank1, n1);
+        InstrumentEntry &instrRef2 = getInstrumentReference(root2, bank2, n2);
         if (instrRef1.name == instrRef2.name)
         {
             // change the name of the second instrument if the name are equal
             instrRef2.name += "2";
         }
-        if (!setname(n2, getname(n2), n1))
-            return false;
-        if (!setname(n1, getname(n1), n2))
-            return false;
+        if (!setname(n2, getname(n2, bank2, root2), n1, bank1, bank2, root1, root2))
+            message = "can't write to " + getname(n2, bank2, root2);
+        if (!setname(n1, getname(n1, bank1, root1), n2, bank1, bank2, root1, root2))
+            message = "can't write to " + getname(n1, bank1, root1);
         InstrumentEntry instrTmp = instrRef1;
         instrRef1 = instrRef2;
         instrRef2 = instrTmp;
     }
-    return true;
+    if (message > "")
+        result = miscMsgPush(message) | 0x1000;
+    return result;
 }
 
 
 // Intelligently moves or swaps banks preserving instrument details
-void Bank::swapbanks(unsigned int firstID, unsigned int secondID)
+unsigned int Bank::swapbanks(unsigned int firstID, unsigned int secondID, size_t firstRoot, size_t secondRoot)
 {
-    if (firstID == secondID)
-    {
-        synth->getRuntime().Log("Nothing to move!");
-        return;
-    }
+    unsigned int result = 0;
+    string message = "";
+    string firstname;
+    string secondname;
 
-    string firstname = getBankName(firstID); // this needs improving
-    string secondname = getBankName(secondID);
-    if (firstname.empty() and secondname.empty())
+    if (firstRoot > 0x7f)
+        firstRoot = currentRootID;
+    if (secondRoot > 0x7f)
+        secondRoot = firstRoot;
+
+    if (firstID == secondID && firstRoot == secondRoot)
     {
-        synth->getRuntime().Log("Nothing to move!");
-        return;
-    }
-    if (secondname.empty())
-    {
-        synth->getRuntime().Log("Moving " + firstname);
-        roots [currentRootID].banks [secondID] = roots [currentRootID].banks [firstID];
-        roots [currentRootID].banks.erase(firstID);
-    }
-    else if (firstname.empty())
-    {
-        synth->getRuntime().Log("Moving " + secondname);
-        roots [currentRootID].banks [firstID] = roots [currentRootID].banks [secondID];
-        roots [currentRootID].banks.erase(secondID);
+        message = "nothing to swap!";
+        result = 0x1000;
     }
     else
     {
-        synth->getRuntime().Log("Swapping " + firstname + " with " + secondname );
-        roots [currentRootID].banks [firstID].dirname = secondname;
-        roots [currentRootID].banks [secondID].dirname = firstname;
-        hints [currentRootID] [secondname] = firstID; // why do we need these?
-        hints [currentRootID] [firstname] = secondID;
-
-        for(int pos = 0; pos < BANK_SIZE; ++ pos)
+        firstname = getBankName(firstID, firstRoot);
+        secondname = getBankName(secondID, secondRoot);
+        if (firstname.empty() && secondname.empty())
         {
-            InstrumentEntry &instrRef_1 = getInstrumentReference(currentRootID, firstID, pos);
-            InstrumentEntry &instrRef_2 = getInstrumentReference(currentRootID, secondID, pos);
-
-            InstrumentEntry tmp = instrRef_2;
-
-            if (instrRef_1.name == "")
-                roots [currentRootID].banks [secondID].instruments.erase(pos);
-            else
-                instrRef_2 = instrRef_1;
-
-            if (tmp.name == "")
-                roots [currentRootID].banks [firstID].instruments.erase(pos);
-            else
-                instrRef_1 = tmp;
+            message = "nothing to swap!";
+            result = 0x1000;
         }
     }
 
-    if (firstID == currentBankID)
-        currentBankID = secondID;
-    else if(secondID == currentBankID)
-        currentBankID = firstID;
+    if (firstRoot != secondRoot)
+    {
+        if (isDuplicateBankName(firstRoot, secondname))
+        {
+            message = secondname + " already exists in root " + to_string(firstRoot);
+            result = 0x1000;
+        }
+        else if (isDuplicateBankName(secondRoot, firstname))
+        {
+            message = firstname + " already exists in root " + to_string(secondRoot);
+            result = 0x1000;
+        }
+    }
+
+    if (result == 0 && firstRoot != secondRoot) // do physical move first
+    {
+        string firstBankPath = getBankPath(firstRoot, firstID);
+        string secondBankPath = getBankPath(secondRoot, secondID);
+        string newfirstBankPath = getRootPath(secondRoot) + "/" + firstname;
+        string newsecondBankPath = getRootPath(firstRoot) + "/" + secondname;
+        string tempBankPath = getRootPath(firstRoot) + "/tempfile";
+        if (secondBankPath == "") // move only
+        {
+            result = rename(firstBankPath.c_str(), (getRootPath(secondRoot) + "/" + firstname).c_str());
+            if (result != 0)
+            {
+                synth->getRuntime().Log("move to " + to_string(secondRoot) + ": " + string(strerror(errno)), 2);
+                message = "Can't move from root " + to_string(firstRoot) + " to " + to_string(secondRoot);
+                result = 0x1000;
+            }
+        }
+        else if(firstBankPath == "") // move only
+        {
+            result = rename(secondBankPath.c_str(), (getRootPath(firstRoot) + "/" + secondname).c_str());
+            if (result != 0)
+            {
+                synth->getRuntime().Log("move to " + to_string(firstRoot) + ": " + string(strerror(errno)), 2);
+                message = "Can't move from root " + to_string(secondRoot) + " to " + to_string(firstRoot);
+                result = 0x1000;
+            }
+        }
+        else // actual swap
+        {
+            // due to possible identical names we need to go via a temp file
+
+            //cout << "first " << firstBankPath << endl;
+            //cout << "second " << secondBankPath << endl;
+            //cout << "newfirst " << newfirstBankPath << endl;
+            //cout << "newsecond " << newsecondBankPath << endl;
+            result = remove(tempBankPath.c_str()); // just to be sure
+            result = rename(firstBankPath.c_str(), tempBankPath.c_str());
+            if (result == 0)
+            {
+                result = rename(secondBankPath.c_str(),newsecondBankPath.c_str());
+                if (result == 0)
+                {
+                    result = rename(tempBankPath.c_str(), newfirstBankPath.c_str());
+                    if (result != 0)
+                    {
+                        synth->getRuntime().Log("move to " + to_string(secondRoot) + ": " + string(strerror(errno)), 2);
+                        message = "Can't move from temp dir to " + to_string(secondRoot);
+                        result = 0x1000;
+                    }
+                }
+                else
+                {
+                    synth->getRuntime().Log("move to " + to_string(firstRoot) + ": " + string(strerror(errno)), 2);
+                    message = "Can't move from root " + to_string(secondRoot) + " to " + to_string(firstRoot);
+                    result = 0x1000;
+                }
+            }
+            else
+            {
+                synth->getRuntime().Log("move to temp dir: " + string(strerror(errno)), 2);
+                message = "Can't move from root " + to_string(firstRoot) + " to temp dir";
+                result = 0x1000;
+            }
+        }
+    }
+
+    if (result == 0) // update banks
+    {
+        if (secondname.empty())
+        {
+            roots [secondRoot].banks [secondID] = roots [firstRoot].banks [firstID];
+            roots [firstRoot].banks.erase(firstID);
+        }
+        else if (firstname.empty())
+        {
+            roots [firstRoot].banks [firstID] = roots [secondRoot].banks [secondID];
+            roots [secondRoot].banks.erase(secondID);
+        }
+        else
+        {
+            roots [firstRoot].banks [firstID].dirname = secondname;
+            roots [secondRoot].banks [secondID].dirname = firstname;
+            hints [secondRoot] [secondname] = firstID; // why do we need these?
+            hints [firstRoot] [firstname] = secondID;
+
+            for(int pos = 0; pos < BANK_SIZE; ++ pos)
+            {
+                InstrumentEntry &instrRef_1 = getInstrumentReference(firstRoot, firstID, pos);
+                InstrumentEntry &instrRef_2 = getInstrumentReference(secondRoot, secondID, pos);
+
+                InstrumentEntry tmp = instrRef_2;
+
+                if (instrRef_1.name == "")
+                    roots [secondRoot].banks [secondID].instruments.erase(pos);
+                else
+                    instrRef_2 = instrRef_1;
+
+                if (tmp.name == "")
+                    roots [firstRoot].banks [firstID].instruments.erase(pos);
+                else
+                    instrRef_1 = tmp;
+            }
+        }
+    }
+
+    if (result == 0)
+    {
+        if (firstRoot == currentRootID)
+            currentRootID = secondRoot;
+        else if(secondRoot == currentBankID)
+            currentBankID = firstRoot;
+        if (firstID == currentBankID)
+            currentBankID = secondID;
+        else if(secondID == currentBankID)
+            currentBankID = firstID;
+    }
+
+    result |= miscMsgPush(message);
+    return result;
 }
 
 
@@ -571,8 +996,6 @@ void Bank::scanrootdir(int root_idx)
             continue;
         }
         struct dirent *fname;
-        //int idx;
-        //char x;
         while ((fname = readdir(d)))
         {
             string possible = string(fname->d_name);
@@ -661,15 +1084,20 @@ bool Bank::addtobank(size_t rootID, size_t bankID, int pos, const string filenam
     instrRef.PADsynth_used = false;
     instrRef.ADDsynth_used = false;
     instrRef.SUBsynth_used = false;
+    instrRef.yoshiType = false;
 
     // see which engines are used
     if (synth->getRuntime().checksynthengines)
     {
-        XMLwrapper *xml = new XMLwrapper(synth);
-        xml->checkfileinformation(getFullPath(rootID, bankID, pos));
+        string checkfile = setExtension(getFullPath(rootID, bankID, pos), xiyext);
+        if (!isRegFile(checkfile))
+            checkfile = setExtension(getFullPath(rootID, bankID, pos), xizext);
+        XMLwrapper *xml = new XMLwrapper(synth, true);
+        xml->checkfileinformation(checkfile);
         instrRef.PADsynth_used = xml->information.PADsynth_used;
         instrRef.ADDsynth_used = xml->information.ADDsynth_used;
         instrRef.SUBsynth_used = xml->information.SUBsynth_used;
+        instrRef.yoshiType = xml->information.yoshiType;
         delete xml;
     }
     return 0;
@@ -732,6 +1160,7 @@ void Bank::addDefaultRootDirs()
         "/usr/share/zynaddsubfx/banks",
         "/usr/local/share/zynaddsubfx/banks",
         string(getenv("HOME")) + "/banks",
+        localPath("/banks"),
         "end"
     };
     int i = 0;
@@ -741,7 +1170,6 @@ void Bank::addDefaultRootDirs()
         addRootDir(bankdirs [i]);
         ++ i;
     }
-    addRootDir(localPath("/banks"));
 
     while ( i >= 0)
     {
@@ -882,7 +1310,8 @@ int Bank::engines_used(unsigned int ninstrument)
 {
     int tmp = getInstrumentReference(ninstrument).ADDsynth_used
             | (getInstrumentReference(ninstrument).SUBsynth_used << 1)
-            | (getInstrumentReference(ninstrument).PADsynth_used << 2);
+            | (getInstrumentReference(ninstrument).PADsynth_used << 2)
+            | (getInstrumentReference(ninstrument).yoshiType << 3);
     return tmp;
 }
 
