@@ -5,7 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2013, Nikita Zlobin
-    Copyright 2014-2017, Will Godfrey & others
+    Copyright 2014-2018, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -23,7 +23,7 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified August 2017
+    Modified May 2018
 */
 
 #include <iostream>
@@ -45,14 +45,11 @@
 
 using namespace std;
 
-#include "Synth/BodyDisposal.h"
 #include "Misc/XMLwrapper.h"
 #include "Misc/SynthEngine.h"
 #include "Misc/Config.h"
 #include "MasterUI.h"
-#include "ConfBuild.cpp"
-
-extern void mainRegisterAudioPort(SynthEngine *s, int portnum);
+#include "ConfBuild.h"
 
 static char prog_doc[] =
     "Yoshimi " YOSHIMI_VERSION ", a derivative of ZynAddSubFX - "
@@ -64,29 +61,29 @@ string argline = "Yoshimi " + (string) YOSHIMI_VERSION + "\nBuild Number " + to_
 const char* argp_program_version = argline.c_str();
 
 static struct argp_option cmd_options[] = {
-    {"alsa-audio",        'A',  "<device>",   1,  "use alsa audio output" },
-    {"alsa-midi",         'a',  "<device>",   1,  "use alsa midi input" },
-    {"define-root",       'D',  "<path>",     0,  "define path to new bank root"},
-    {"buffersize",        'b',  "<size>",     0,  "set internal buffer size" },
-    {"no-gui",            'i',  NULL,         0,  "disable gui"},
-    {"gui",               'I',  NULL,         0,  "enable gui"},
-    {"no-cmdline",        'c',  NULL,         0,  "disable command line interface"},
-    {"cmdline",           'C',  NULL,         0,  "enable command line interface"},
-    {"jack-audio",        'J',  "<server>",   1,  "use jack audio output" },
-    {"jack-midi",         'j',  "<device>",   1,  "use jack midi input" },
-    {"autostart-jack",    'k',  NULL,         0,  "auto start jack server" },
-    {"auto-connect",      'K',  NULL,         0,  "auto connect jack audio" },
-    {"load",              'l',  "<file>",     0,  "load .xmz file" },
-    {"load-instrument",   'L',  "<file>",     0,  "load .xiz file" },
-    {"name-tag",          'N',  "<tag>",      0,  "add tag to clientname" },
-    {"samplerate",        'R',  "<rate>",     0,  "set alsa audio sample rate" },
-    {"oscilsize",         'o',  "<size>",     0,  "set AddSynth oscilator size" },
-    {"state",             'S',  "<file>",     1,  "load saved state, defaults to '$HOME/.config/yoshimi/yoshimi.state'" },
+    {"alsa-audio",        'A',  "<device>",   1,  "use alsa audio output", 0},
+    {"alsa-midi",         'a',  "<device>",   1,  "use alsa midi input", 0},
+    {"define-root",       'D',  "<path>",     0,  "define path to new bank root" , 0},
+    {"buffersize",        'b',  "<size>",     0,  "set internal buffer size", 0 },
+    {"no-gui",            'i',  NULL,         0,  "disable gui", 0},
+    {"gui",               'I',  NULL,         0,  "enable gui", 0},
+    {"no-cmdline",        'c',  NULL,         0,  "disable command line interface", 0},
+    {"cmdline",           'C',  NULL,         0,  "enable command line interface", 0},
+    {"jack-audio",        'J',  "<server>",   1,  "use jack audio output", 0},
+    {"jack-midi",         'j',  "<device>",   1,  "use jack midi input", 0},
+    {"autostart-jack",    'k',  NULL,         0,  "auto start jack server", 0},
+    {"auto-connect",      'K',  NULL,         0,  "auto connect jack audio", 0},
+    {"load",              'l',  "<file>",     0,  "load .xmz file", 0},
+    {"load-instrument",   'L',  "<file>",     0,  "load .xiz file", 0},
+    {"name-tag",          'N',  "<tag>",      0,  "add tag to clientname", 0},
+    {"samplerate",        'R',  "<rate>",     0,  "set alsa audio sample rate", 0},
+    {"oscilsize",         'o',  "<size>",     0,  "set AddSynth oscilator size", 0},
+    {"state",             'S',  "<file>",     1,  "load saved state, defaults to '$HOME/.config/yoshimi/yoshimi.state'", 0},
     #if defined(JACK_SESSION)
-        {"jack-session-uuid", 'U',  "<uuid>",     0,  "jack session uuid" },
-        {"jack-session-file", 'u',  "<file>",     0,  "load named jack session file" },
+        {"jack-session-uuid", 'U',  "<uuid>",     0,  "jack session uuid", 0},
+        {"jack-session-file", 'u',  "<file>",     0,  "load named jack session file", 0},
     #endif
-    { 0, }
+    { 0, 0, 0, 0, 0, 0}
 };
 
 unsigned int Config::Samplerate = 48000;
@@ -96,11 +93,14 @@ unsigned int Config::GzipCompression = 3;
 bool         Config::showGui = true;
 bool         Config::showSplash = true;
 bool         Config::showCLI = true;
+bool         Config::autoInstance = false;
+unsigned int Config::activeInstance = 0;
 
 Config::Config(SynthEngine *_synth, int argc, char **argv) :
     restoreState(false),
     stateChanged(false),
     restoreJackSession(false),
+    oldConfig(false),
     runSynth(true),
     finishedCLI(true),
     VirKeybLayout(0),
@@ -118,6 +118,7 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     Interpolation(0),
     checksynthengines(1),
     xmlType(0),
+    instrumentFormat(1),
     EnableProgChange(1), // default will be inverted
     toConsole(0),
     hideErrors(0),
@@ -130,20 +131,20 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     midi_bank_C(32),
     midi_upper_voice_C(128),
     enable_part_on_voice_load(1),
+    enable_NRPN(true),
     ignoreResetCCs(false),
     monitorCCin(false),
     showLearnedCC(true),
     single_row_panel(1),
     NumAvailableParts(NUM_MIDI_CHANNELS),
     currentPart(0),
-    lastPatchSet(-1),
+    VUcount(0),
     channelSwitchType(0),
     channelSwitchCC(128),
     channelSwitchValue(0),
     nrpnL(127),
     nrpnH(127),
     nrpnActive(false),
-    deadObjects(NULL),
     sigIntActive(0),
     ladi1IntActive(0),
     sse_level(0),
@@ -164,7 +165,6 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     //which befaves exactly the same when flag FE_TOWARDZERO is set
 
     cerr.precision(4);
-    deadObjects = new BodyDisposal();
     bRuntimeSetupCompleted = Setup(argc, argv);
 }
 
@@ -228,28 +228,25 @@ bool Config::Setup(int argc, char **argv)
     Buffersize = nearestPowerOf2(Buffersize, 16, 4096);
     //Log(asString(Oscilsize));
     //Log(asString(Buffersize));
-    if (loadDefaultState && !restoreState)
-    {
-        StateFile = ConfigDir + "/yoshimi.state";
-        restoreState = true;
-    }
+
     if (restoreState)
     {
-        char * fp;
-        if (! StateFile.size()) goto no_state0;
-        else fp = new char [PATH_MAX];
+        char *fp = NULL;
+        if (!StateFile.size())
+            goto no_state;
+
+        fp = realpath (StateFile.c_str(), NULL);
+        if (fp == NULL)
+            goto no_state;
 
-        if (! realpath (StateFile.c_str(), fp)) goto no_state1;
         StateFile = fp;
-        delete (fp);
-
-        if (! isRegFile(StateFile))
+        free (fp);
+        if (!isRegFile(StateFile))
         {
-            no_state1: delete (fp);
-            no_state0: Log("Invalid state file specified for restore " + StateFile, 2);
+            no_state: Log("Invalid state file specified for restore " + StateFile, 2);
             return true;
         }
-        Log(StateFile);
+        Log("Using " + StateFile);
         restoreSessionData(StateFile, true);
         /* There is a single state file that contains both startup config
          * data that must be set early, and runtime data that must be set
@@ -417,7 +414,9 @@ bool Config::loadConfig(void)
     string homedir = string(getenv("HOME"));
     if (homedir.empty() || !isDirectory(homedir))
         homedir = string("/tmp");
+    userHome = homedir + '/';
     ConfigDir = homedir + string("/.config/") + YOSHIMI;
+    defaultStateName = ConfigDir + "/yoshimi";
     if (!isDirectory(ConfigDir))
     {
         cmd = string("mkdir -p ") + ConfigDir;
@@ -427,11 +426,12 @@ bool Config::loadConfig(void)
             return false;
         }
     }
-    string yoshimi = "/"; // for some reason it doesn't
-    yoshimi += YOSHIMI; // like these as one line here
+    string yoshimi = "/" + string(YOSHIMI);
 
-    if (synth->getUniqueId() > 0)
-        yoshimi += ("-" + asString(synth->getUniqueId()));
+    string baseConfig = ConfigDir + yoshimi + ".config";
+    int thisInstance = synth->getUniqueId();
+    if (thisInstance > 0)
+        yoshimi += ("-" + asString(thisInstance));
     else
         miscMsgInit(); // sneaked it in here so it's early
     string presetDir = ConfigDir + "/presets";
@@ -441,43 +441,61 @@ bool Config::loadConfig(void)
         if ((chk = system(cmd.c_str())) < 0)
             Log("Create preset directory " + presetDir + " failed, status " + asString(chk));
     }
+
     ConfigFile = ConfigDir + yoshimi;
     StateFile = ConfigDir + yoshimi + string(".state");
-    if (synth->getUniqueId() == 0)
-        ConfigFile += ".config";
+
+    if (thisInstance == 0)
+        ConfigFile = baseConfig;
     else
         ConfigFile += ".instance";
-    string resConfigFile = ConfigFile;
 
+    if (!isRegFile(baseConfig))
+    {
+        Log("Basic configuration " + baseConfig + " not found, will use default settings");
+        defaultPresets();
+    }
 
     bool isok = true;
-    if (!isRegFile(resConfigFile) && !isRegFile(ConfigFile))
+    if (!isRegFile(ConfigFile))
     {
-        Log("ConfigFile " + resConfigFile + " not found, will use default settings");
-        defaultPresets();
+        Log("Configuration " + ConfigFile + " not found, will use default settings");
         configChanged = true; // give the user the choice
     }
     else
     {
-        XMLwrapper *xml = new XMLwrapper(synth);
+        XMLwrapper *xml = new XMLwrapper(synth, true);
         if (!xml)
             Log("loadConfig failed XMLwrapper allocation");
         else
         {
-            if (!xml->loadXMLfile(resConfigFile))
+            if (!xml->loadXMLfile(baseConfig))
             {
-                if ((synth->getUniqueId() > 0) && (!xml->loadXMLfile(ConfigFile)))
+                if (thisInstance > 0)
                 {
                     Log("loadConfig loadXMLfile failed");
                     return false;
                 }
             }
             isok = extractBaseParameters(xml);
-            if (isok)
-                isok = extractConfigData(xml);
-            if (isok)
-                Oscilsize = (int)truncf(powf(2.0f, ceil(log (Oscilsize - 1.0f) / logf(2.0))));
             delete xml;
+            if (isok)
+            {
+                XMLwrapper *xml = new XMLwrapper(synth, true);
+                isok = xml->loadXMLfile(ConfigFile);
+                if (isok)
+                {
+                    isok = extractConfigData(xml);
+                    delete xml;
+                }
+            }
+            if (thisInstance == 0)
+            {
+                if (lastXMLmajor < MIN_CONFIG_MAJOR || lastXMLminor < MIN_CONFIG_MINOR)
+                    oldConfig = true;
+                else
+                    oldConfig = false;
+            }
         }
     }
     return isok;
@@ -530,11 +548,8 @@ bool Config::extractBaseParameters(XMLwrapper *xml)
     showGui = xml->getparbool("enable_gui", showGui);
     showSplash = xml->getparbool("enable_splash", showSplash);
     showCLI = xml->getparbool("enable_CLI", showCLI);
-    /*if (!showGui && !showCLI)
-    {
-        showGui = true;
-        showCLI = true; // sanity check!
-    }*/
+    autoInstance = xml->getparbool("enable_auto_instance", autoInstance);
+    activeInstance = xml->getparU("active_instances", 0);
     xml->exitbranch(); // BaseParameters
     return true;
 }
@@ -604,6 +619,8 @@ bool Config::extractConfigData(XMLwrapper *xml)
     midi_upper_voice_C = xml->getpar("midi_upper_voice_C", midi_upper_voice_C, 0, 128);
     EnableProgChange = 1 - xml->getpar("ignore_program_change", EnableProgChange, 0, 1); // inverted for Zyn compatibility
     enable_part_on_voice_load = xml->getpar("enable_part_on_voice_load", enable_part_on_voice_load, 0, 1);
+    instrumentFormat = xml->getpar("saved_instrument_format",instrumentFormat, 1, 3);
+    enable_NRPN = xml->getparbool("enable_incoming_NRPNs", enable_NRPN);
     ignoreResetCCs = xml->getpar("ignore_reset_all_CCs",ignoreResetCCs,0, 1);
     monitorCCin = xml->getparbool("monitor-incoming_CCs", monitorCCin);
     showLearnedCC = xml->getparbool("open_editor_on_learned_CC", showLearnedCC);
@@ -622,7 +639,7 @@ bool Config::saveConfig(void)
 {
     bool result = false;
     xmlType = XML_CONFIG;
-    XMLwrapper *xmltree = new XMLwrapper(synth);
+    XMLwrapper *xmltree = new XMLwrapper(synth, true);
     if (!xmltree)
     {
         Log("saveConfig failed xmltree allocation", 2);
@@ -682,6 +699,8 @@ void Config::addConfigXML(XMLwrapper *xmltree)
     xmltree->addpar("midi_upper_voice_C", midi_upper_voice_C);
     xmltree->addpar("ignore_program_change", (1 - EnableProgChange));
     xmltree->addpar("enable_part_on_voice_load", enable_part_on_voice_load);
+    xmltree->addpar("saved_instrument_format", instrumentFormat);
+    xmltree->addparbool("enable_incoming_NRPNs", enable_NRPN);
     xmltree->addpar("ignore_reset_all_CCs",ignoreResetCCs);
     xmltree->addparbool("monitor-incoming_CCs", monitorCCin);
     xmltree->addparbool("open_editor_on_learned_CC",showLearnedCC);
@@ -696,7 +715,7 @@ bool Config::saveSessionData(string savefile)
 {
     savefile = setExtension(savefile, "state");
     synth->getRuntime().xmlType = XML_STATE;
-    XMLwrapper *xmltree = new XMLwrapper(synth);
+    XMLwrapper *xmltree = new XMLwrapper(synth, true);
     if (!xmltree)
     {
         Log("saveSessionData failed xmltree allocation", 3);
@@ -731,7 +750,7 @@ bool Config::restoreSessionData(string sessionfile, bool startup)
         Log("Session file " + sessionfile + " not available", 2);
         goto end_game;
     }
-    if (!(xml = new XMLwrapper(synth)))
+    if (!(xml = new XMLwrapper(synth, true)))
     {
         Log("Failed to init xmltree for restoreState", 3);
         goto end_game;
@@ -771,7 +790,7 @@ end_game:
 }
 
 
-void Config::Log(string msg, char tostderr)
+void Config::Log(const string &msg, char tostderr)
 {
     if ((tostderr & 2) && hideErrors)
         return;
@@ -784,6 +803,10 @@ void Config::Log(string msg, char tostderr)
         cerr << msg << endl; // error log
 }
 
+void Config::LogError(const string &msg)
+{
+    Log("[ERROR] " + msg, 1);
+}
 
 #ifndef YOSHIMI_LV2_PLUGIN
 void Config::StartupReport(MusicClient *musicClient)
@@ -1208,14 +1231,13 @@ static error_t parse_cmds (int key, char *arg, struct argp_state *state)
             break;
 
         default:
-            return ARGP_ERR_UNKNOWN;
+            return error_t(ARGP_ERR_UNKNOWN);
     }
-
-    return 0;
+    return error_t(0);
 }
 
 
-static struct argp cmd_argp = { cmd_options, parse_cmds, prog_doc };
+static struct argp cmd_argp = { cmd_options, parse_cmds, prog_doc, 0, 0, 0, 0};
 
 
 void Config::loadCmdArgs(int argc, char **argv)
@@ -1232,14 +1254,6 @@ void GuiThreadMsg::processGuiMessages()
     if (msg)
     {
         SynthEngine *synth = ((SynthEngine *)msg->data);
-
-        if (msg->type == RegisterAudioPort)
-        {
-            // This is unique not using guiMaster
-             mainRegisterAudioPort(synth, msg->index);
-             delete msg;
-             return;
-        }
         MasterUI *guiMaster = synth->getGuiMaster((msg->type == GuiThreadMsg::NewSynthEngine));
         if (msg->type == GuiThreadMsg::NewSynthEngine)
         {
@@ -1267,51 +1281,11 @@ void GuiThreadMsg::processGuiMessages()
                     guiMaster->updatepaths(msg->index);
                     break;
 
-                case GuiThreadMsg::UpdatePanel:
-                    guiMaster->updatepanel();
-                    break;
-
                 case GuiThreadMsg::UpdatePart:
                     guiMaster->updatepart();
                     guiMaster->updatepanel();
                     break;
 
-                case GuiThreadMsg::UpdatePanelItem:
-                    if ( msg->data && msg->index < NUM_MIDI_PARTS)
-                    {
-                        guiMaster->updatelistitem(msg->index);
-                        guiMaster->updatepart();
-                    }
-                    break;
-
-                case GuiThreadMsg::UpdatePartProgram:
-                    if (msg->data && msg->index < NUM_MIDI_PARTS)
-                    {
-                        guiMaster->updatelistitem(msg->index);
-                        guiMaster->updatepartprogram(msg->index);
-                    }
-                    break;
-
-                case GuiThreadMsg::UpdateEffects:
-                    if (msg->data)
-                        guiMaster->updateeffects(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdateControllers:
-                    if (msg->data)
-                        guiMaster->updatecontrollers(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdateBankRootDirs:
-                    if (msg->data)
-                        guiMaster->updateBankRootDirs();
-                    break;
-
-                case GuiThreadMsg::RescanForBanks:
-                    if (msg->data && guiMaster->bankui)
-                        guiMaster->bankui->rescan_for_banks(false);
-                    break;
-
                 case GuiThreadMsg::RefreshCurBank:
                     if (msg->data && guiMaster->bankui)
                     {
