@@ -2,7 +2,7 @@
     main.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2017, Will Godfrey & others
+    Copyright 2014-2018, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,14 +17,19 @@
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 
-    Modified May 2017
+    Modified May 2018
 */
 
+// approx timeout in seconds.
+#define SPLASH_TIME 3
+
 #include <sys/mman.h>
 #include <iostream>
 #include <stdio.h>
 #include <sys/types.h>
 #include <termios.h>
+#include <time.h>
+
 
 using namespace std;
 
@@ -34,7 +39,6 @@ using namespace std;
 #include "MusicIO/MusicClient.h"
 #include "MasterUI.h"
 #include "UI/MiscGui.h"
-#include "Synth/BodyDisposal.h"
 #include <map>
 #include <list>
 #include <pthread.h>
@@ -44,7 +48,6 @@ using namespace std;
 
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
-#include <FL/Fl_Shared_Image.H>
 #include <FL/Fl_PNG_Image.H>
 
 #include <readline/readline.h>
@@ -53,18 +56,19 @@ using namespace std;
 
 CmdInterface commandInt;
 
+extern map<SynthEngine *, MusicClient *> synthInstances;
+extern SynthEngine *firstSynth;
+
 void mainRegisterAudioPort(SynthEngine *s, int portnum);
+int mainCreateNewInstance(unsigned int forceId, bool loadState);
 
-map<SynthEngine *, MusicClient *> synthInstances;
-list<string> splashMessages;
-
-static SynthEngine *firstSynth = NULL;
-static Config *firstRuntime = NULL;
+Config *firstRuntime = NULL;
 static int globalArgc = 0;
 static char **globalArgv = NULL;
 bool bShowGui = true;
 bool bShowCmdLine = true;
-
+bool splashSet = true;
+time_t old_father_time, here_and_now;
 
 //Andrew Deryabin: signal handling moved to main from Config Runtime
 //It's only suitable for single instance app support
@@ -91,24 +95,19 @@ void yoshimiSigHandler(int sig)
     }
 }
 
-void splashTimeout(void *splashWin)
-{
-    (static_cast<Fl_Window *>(splashWin))->hide();
-}
-
 static void *mainGuiThread(void *arg)
 {
+    Fl::lock();
+
     sem_post((sem_t *)arg);
 
     map<SynthEngine *, MusicClient *>::iterator it;
-    fl_register_images();
 
     const int textHeight = 15;
     const int textY = 10;
     const unsigned char lred = 0xd7;
     const unsigned char lgreen = 0xf7;
     const unsigned char lblue = 0xff;
-    const float timeout = 3.5f;
 
     Fl_PNG_Image pix("splash_screen_png", splashPngData, splashPngLength);
     Fl_Window winSplash(splashWidth, splashHeight, "yoshimi splash screen");
@@ -123,19 +122,13 @@ static void *mainGuiThread(void *arg)
     boxLb.labeltype(FL_NORMAL_LABEL);
     boxLb.labelcolor(fl_rgb_color(lred, lgreen, lblue));
     boxLb.labelfont(FL_HELVETICA | FL_BOLD);
-    // see later!
-    //winSplash.set_modal();
-    //winSplash.clear_border();
     winSplash.border(false);
-    bool splashSet = false;
-    if (bShowGui && firstRuntime->showSplash)
+    if (splashSet && bShowGui && firstRuntime->showSplash)
     {
-        splashSet = true;
         winSplash.position((Fl::w() - winSplash.w()) / 2, (Fl::h() - winSplash.h()) / 2);
-        winSplash.show();
-        Fl::add_timeout(timeout, splashTimeout, &winSplash);
     }
-
+    else
+        splashSet = false;
     do
     {
             usleep(33333);
@@ -144,18 +137,25 @@ static void *mainGuiThread(void *arg)
 
     GuiThreadMsg::sendMessage(firstSynth, GuiThreadMsg::NewSynthEngine, 0);
 
-    while (firstSynth->getRuntime().runSynth)
+    if (firstRuntime->autoInstance)
+    {
+        for (int i = 1; i < 32; ++i)
+        {
+            if ((firstRuntime->activeInstance >> i) & 1)
+                mainCreateNewInstance(i, true);
+        }
+    }
+    while (firstRuntime->runSynth)
     {
         if (firstSynth->getUniqueId() == 0)
         {
-            firstSynth->getRuntime().signalCheck();
+            firstRuntime->signalCheck();
         }
 
         for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
         {
             SynthEngine *_synth = it->first;
             MusicClient *_client = it->second;
-            _synth->getRuntime().deadObjects->disposeBodies();
             if (!_synth->getRuntime().runSynth && _synth->getUniqueId() > 0)
             {
                 if (_synth->getRuntime().configChanged)
@@ -163,11 +163,10 @@ static void *mainGuiThread(void *arg)
                     size_t tmpRoot = _synth->ReadBankRoot();
                     size_t tmpBank = _synth->ReadBank();
                     _synth->getRuntime().loadConfig(); // restore old settings
-                    _synth->SetBankRoot(tmpRoot);
-                    _synth->SetBank(tmpBank); // but keep current root and bank
+                    _synth->RootBank(tmpRoot, tmpBank); // but keep current root and bank
                 }
                 _synth->getRuntime().saveConfig();
-                int tmpID =  _synth->getUniqueId();
+                unsigned int instanceID =  _synth->getUniqueId();
                 if (_client)
                 {
                     _client->Close();
@@ -176,14 +175,16 @@ static void *mainGuiThread(void *arg)
 
                 if (_synth)
                 {
-                    _synth->saveBanks(tmpID);
-                    _synth->getRuntime().deadObjects->disposeBodies();
+                    int instancebit = (1 << instanceID);
+                    if (_synth->getRuntime().activeInstance & instancebit)
+                        _synth->getRuntime().activeInstance -= instancebit;
+                    _synth->saveBanks(instanceID);
                     _synth->getRuntime().flushLog();
                     delete _synth;
                 }
 
                 synthInstances.erase(it);
-                cout << "\nStopped " << tmpID << "\n";
+                cout << "\nStopped " << instanceID << "\n";
                 break;
             }
             if (bShowGui)
@@ -192,18 +193,7 @@ static void *mainGuiThread(void *arg)
                 {
                     MasterUI *guiMaster = _synth->getGuiMaster(false);
                     if (guiMaster)
-                    {
-/*
- * this hack is necessary because 'set_non_modal' doesn't work
- * on all WMs, and 'set_modal' stops the user doing anything
- * while the splash is visible
- */
-                        if (i == 0 && splashSet == true)
-                        {
-                            winSplash.show();
-                            splashSet = false;
-                        }
-                        guiMaster->Log(_synth->getRuntime().LogList.front());
+                    { guiMaster->Log(_synth->getRuntime().LogList.front());
                         _synth->getRuntime().LogList.pop_front();
                     }
                 }
@@ -213,33 +203,46 @@ static void *mainGuiThread(void *arg)
         // where all the action is ...
         if (bShowGui)
         {
+            if (splashSet)
+            {
+                winSplash.show();
+                usleep(1000);
+                if(time(&here_and_now) < 0) // no time?
+                    here_and_now = old_father_time + SPLASH_TIME;
+                if ((here_and_now - old_father_time) >= SPLASH_TIME)
+                {
+                    splashSet = false;
+                    winSplash.hide();
+                }
+            }
             Fl::wait(0.033333);
             GuiThreadMsg::processGuiMessages();
         }
         else
             usleep(33333);
     }
-    if (firstSynth->getRuntime().configChanged && (bShowGui | bShowCmdLine)) // don't want this if no cli or gui
+    if (firstRuntime->configChanged && (bShowGui | bShowCmdLine)) // don't want this if no cli or gui
     {
         size_t tmpRoot = firstSynth->ReadBankRoot();
         size_t tmpBank = firstSynth->ReadBank();
-        firstSynth->getRuntime().loadConfig(); // restore old settings
-        firstSynth->SetBankRoot(tmpRoot);
-        firstSynth->SetBank(tmpBank); // but keep current root and bank
+        firstRuntime->loadConfig(); // restore old settings
+        firstSynth->RootBank(tmpRoot, tmpBank); // but keep current root and bank
     }
-    firstSynth->getRuntime().saveConfig();
+
+    firstRuntime->saveConfig();
     firstSynth->saveHistory();
     firstSynth->saveBanks(0);
     return NULL;
 }
 
-bool mainCreateNewInstance(unsigned int forceId)
+int mainCreateNewInstance(unsigned int forceId, bool loadState)
 {
     MusicClient *musicClient = NULL;
+    unsigned int instanceID;
     SynthEngine *synth = new SynthEngine(globalArgc, globalArgv, false, forceId);
     if (!synth->getRuntime().isRuntimeSetupCompleted())
         goto bail_out;
-
+    instanceID = synth->getUniqueId();
     if (!synth)
     {
         std::cerr << "Failed to allocate SynthEngine" << std::endl;
@@ -263,6 +266,16 @@ bool mainCreateNewInstance(unsigned int forceId)
         synth->getRuntime().Log("Failed to start MusicIO");
         goto bail_out;
     }
+     // TODO sort this out properly!
+     // it works, but is clunky :(
+    loadState = synth->getRuntime().loadDefaultState;
+    if (loadState)
+    {
+        string name = synth->getRuntime().defaultStateName;
+        if (instanceID > 0)
+            name = name + "-" + to_string(forceId);
+        synth->loadStateAndUpdate(name);
+    }
 
     if (synth->getRuntime().showGui)
     {
@@ -279,13 +292,14 @@ bool mainCreateNewInstance(unsigned int forceId)
 
     synth->getRuntime().StartupReport(musicClient);
     synth->Unmute();
-    if (synth->getUniqueId() == 0)
+
+    if (instanceID == 0)
         cout << "\nYay! We're up and running :-)\n";
     else
     {
-        cout << "\nStarted "<< synth->getUniqueId() << "\n";
+        cout << "\nStarted "<< instanceID << "\n";
         // following copied here for other instances
-        synth->installBanks(synth->getUniqueId());
+        synth->installBanks(instanceID);
     }
     synthInstances.insert(std::make_pair(synth, musicClient));
     //register jack ports for enabled parts
@@ -294,7 +308,8 @@ bool mainCreateNewInstance(unsigned int forceId)
         if (synth->partonoffRead(npart))
             mainRegisterAudioPort(synth, npart);
     }
-    return true;
+    synth->getRuntime().activeInstance |= (1 << instanceID);
+    return instanceID;
 
 bail_out:
     synth->getRuntime().runSynth = false;
@@ -310,10 +325,10 @@ bail_out:
         delete synth;
     }
 
-    return false;
+    return -1;
 }
 
-void *commandThread(void *arg)
+void *commandThread(void *arg = NULL) // silence warning
 {
     commandInt.cmdIfaceCommandLoop();
     return 0;
@@ -321,6 +336,8 @@ void *commandThread(void *arg)
 
 int main(int argc, char *argv[])
 {
+    time(&old_father_time);
+    here_and_now = old_father_time;
     struct termios  oldTerm;
     tcgetattr(0, &oldTerm);
 
@@ -334,10 +351,7 @@ int main(int argc, char *argv[])
     pthread_attr_t attr;
     sem_t semGui;
 
-    // moved from mainGuiThread() to prevent leaking from early GuiThreadMessage
-    Fl::lock();
-
-    if (!mainCreateNewInstance(0))
+    if (mainCreateNewInstance(0, false) == -1)
     {
         goto bail_out;
     }
@@ -347,14 +361,12 @@ int main(int argc, char *argv[])
     firstSynth = it->first;
     bShowGui = firstRuntime->showGui;
     bShowCmdLine = firstRuntime->showCLI;
-    /*if (!(bShowGui | bShowCmdLine))
-    {
-        cout << "Can't disable both gui and command line!\nSet for command line.\n";
-        firstRuntime->showCLI = true;
-        bShowCmdLine = true;
-        firstRuntime->configChanged = true;
-    }*/
 
+    if (firstRuntime->oldConfig)
+    {
+
+        cout << "\nExisting config older than " << MIN_CONFIG_MAJOR << "." << MIN_CONFIG_MINOR << "\nCheck settings, save and restart.\n"<< endl;
+    }
     if(sem_init(&semGui, 0, 0) == 0)
     {
         if (pthread_attr_init(&attr) == 0)
@@ -378,15 +390,15 @@ int main(int argc, char *argv[])
     memset(&yoshimiSigAction, 0, sizeof(yoshimiSigAction));
     yoshimiSigAction.sa_handler = yoshimiSigHandler;
     if (sigaction(SIGUSR1, &yoshimiSigAction, NULL))
-        firstSynth->getRuntime().Log("Setting SIGUSR1 handler failed");
+        firstRuntime->Log("Setting SIGUSR1 handler failed");
     if (sigaction(SIGINT, &yoshimiSigAction, NULL))
-        firstSynth->getRuntime().Log("Setting SIGINT handler failed");
+        firstRuntime->Log("Setting SIGINT handler failed");
     if (sigaction(SIGHUP, &yoshimiSigAction, NULL))
-        firstSynth->getRuntime().Log("Setting SIGHUP handler failed");
+        firstRuntime->Log("Setting SIGHUP handler failed");
     if (sigaction(SIGTERM, &yoshimiSigAction, NULL))
-        firstSynth->getRuntime().Log("Setting SIGTERM handler failed");
+        firstRuntime->Log("Setting SIGTERM handler failed");
     if (sigaction(SIGQUIT, &yoshimiSigAction, NULL))
-        firstSynth->getRuntime().Log("Setting SIGQUIT handler failed");
+        firstRuntime->Log("Setting SIGQUIT handler failed");
     // following moved here for faster first synth startup
     firstSynth->loadHistory();
     firstSynth->installBanks(0);
@@ -394,6 +406,7 @@ int main(int argc, char *argv[])
 
     //create command line processing thread
     pthread_t cmdThr;
+//    while (firstSynth == NULL); // just wait
     if(bShowCmdLine)
     {
         if (pthread_attr_init(&attr) == 0)
@@ -416,8 +429,6 @@ int main(int argc, char *argv[])
     bExitSuccess = true;
 
 bail_out:
-    if (bShowGui && !bExitSuccess) // this could be done better!
-        sleep(2);
     for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
     {
         SynthEngine *_synth = it->first;
@@ -436,7 +447,6 @@ bail_out:
 
         if (_synth)
         {
-            _synth->getRuntime().deadObjects->disposeBodies();
             _synth->getRuntime().flushLog();
             delete _synth;
         }
